@@ -18,7 +18,7 @@ States https://optuna.readthedocs.io/en/stable/reference/generated/optuna.trial.
 """
 from pathlib import Path
 import shutil
-import uuid
+from uuid import uuid4
 import logging
 import os
 from itertools import combinations
@@ -61,8 +61,10 @@ class Optuna(Optimize):
                  parameters=None, objectives=None, constraints=None,
                  sampler=None, sampler_kwargs=None,
                  pruner=None, pruner_kwargs=None,
-                 do_results=None, do_csv=None, do_excel=None, do_plot=None,
-                 do_plot_pareto=None, do_plot_other=None,
+                 do_results=None, do_csv=None, do_excel=None,
+                 do_plot=None, do_plot_pareto=None, do_plot_history=None,
+                 do_plot_slice=None, do_plot_contour=None, do_plot_parallel=None,
+                 do_plot_edf=None, do_plot_importance=None,
                  results_color_key=None,
                  results_color_scale=None,
                  do_results_color_reverse=None,
@@ -72,9 +74,11 @@ class Optuna(Optimize):
                  **kwargs):
         super().__init__(**kwargs)
         # Study
-        self.storage = storage
-        self.study = str(uuid.uuid4()) if study is None else study
-        self.work_path = Path(str(uuid.uuid4())).resolve() if work_path is None else Path(work_path).resolve()
+        self.storage = os.getenv('OPTUNA_URL', None) if storage is None else storage
+        self.study = os.getenv('OPTUNA_STUDY', str(uuid4())) if study is None else study
+        if work_path is None:
+            work_path = os.getenv('OPTUNA_WORKDIR', str(uuid4()))
+        self.work_path = Path(work_path).resolve()
         self.study_path = (self.work_path / self.study).resolve()
         self.do_create_study = True if do_create_study is None else do_create_study
         self.do_read_study = True if do_read_study is None else do_read_study
@@ -96,9 +100,14 @@ class Optuna(Optimize):
         self.do_results = True if do_results is None else do_results
         self.do_csv = True if do_csv is None else do_csv
         self.do_excel = False if do_excel is None else do_excel
-        self.do_plot = False if do_plot is None else do_plot
-        self.do_plot_pareto = True if do_plot_pareto is None else do_plot_pareto
-        self.do_plot_other = True if do_plot_other is None else do_plot_other
+        self.do_plot = True if do_plot is None else do_plot
+        self.do_plot_pareto = False if do_plot_pareto is None else do_plot_pareto
+        self.do_plot_history = False if do_plot_history is None else do_plot_history
+        self.do_plot_slice = False if do_plot_slice is None else do_plot_slice
+        self.do_plot_contour = False if do_plot_contour is None else do_plot_contour
+        self.do_plot_parallel = False if do_plot_parallel is None else do_plot_parallel
+        self.do_plot_edf = False if do_plot_edf is None else do_plot_edf
+        self.do_plot_importance = False if do_plot_importance is None else do_plot_importance
         self.results_color_key = results_color_key
         self.results_color_scale = 'Viridis' if results_color_scale is None else results_color_scale
         self.do_results_reverse_color = False if do_results_color_reverse is None else do_results_color_reverse
@@ -290,10 +299,12 @@ class Optuna(Optimize):
                 if kind == 'suggested':
                     if isinstance(t.low, int) and isinstance(t.high, int):
                         s = (t.high - t.low) // (t.num - 1) if t.num != 1 else 1
-                        value = trial.suggest_int(name=r, low=t.low, high=t.high, step=s)
+                        value = trial.suggest_int(name=r, low=t.low, high=t.high,
+                                                  step=s)
                     else:
                         s = (t.high - t.low) / (t.num - 1) if t.num != 1 else 1
-                        value = trial.suggest_float(name=r, low=t.low, high=t.high, step=s)
+                        value = trial.suggest_float(name=r, low=t.low, high=t.high,
+                                                    step=s)
                     v.low, v.high, v.num = value, value, 1
                 elif kind == 'optimized':
                     value = trial.user_attrs[f'features_{r}']
@@ -317,15 +328,100 @@ class Optuna(Optimize):
                 raise ValueError('Optuna works with Continuous, Discrete '
                                  'and Categorical variables only!')
 
-    def create_study(self):
+    def initialize_sampler(self):
+        sampler = None
         if self.sampler is not None:
+            if self.sampler == 'PartialFixedSampler':  # Initialize base sampler
+                if 'base_sampler' in self.sampler_kwargs:
+                    bs_str = self.sampler_kwargs['base_sampler']
+                else:
+                    logging.warning('base_sampler is not in sampler_kwargs '
+                                    '- using RandomSampler')
+                    bs_str = 'RandomSampler'
+                if 'base_sampler_kwargs' in self.sampler_kwargs:
+                    ks = self.sampler_kwargs.pop('base_sampler_kwargs')
+                else:
+                    logging.warning(f'base_sampler_kwargs is not in sampler_kwargs '
+                                    f'- using default parameters of {bs_str}')
+                    ks = {}
+                bs = getattr(optuna.samplers, bs_str)(**ks)
+                self.sampler_kwargs['base_sampler'] = bs
+                if 'fixed_params' in self.sampler_kwargs:
+                    ps = self.sampler_kwargs['fixed_params']
+                else:
+                    logging.warning(f'fixed_params is not in sampler_kwargs '
+                                    f'- sampling with {bs_str} without restrictions')
+                    ps = self.sampler_kwargs.setdefault('fixed_params', {})
+                s = None  # study
+                for k in list(ps.keys()):
+                    v = ps[k]
+                    if v is None:
+                        if s is None:
+                            if len(self.objectives) == 1:
+                                direction = list(self.objectives.values())[0]
+                                s = optuna.create_study(storage=self.storage,
+                                                        study_name=self.study,
+                                                        load_if_exists=True,
+                                                        direction=direction)
+                            elif len(self.objectives) > 1:  # Multi-objective
+                                directions = list(self.objectives.values())
+                                s = optuna.create_study(storage=self.storage,
+                                                        study_name=self.study,
+                                                        load_if_exists=True,
+                                                        directions=directions)
+                            else:
+                                raise ValueError(f'No objectives: {self.objectives}!')
+                        if len(s.trials) > 0:
+                            if not s._is_multi_objective:
+                                t = s.best_trial
+                            else:
+                                t = s.best_trials[0]
+                            b = t.params[k]
+                            logging.info(f'Fixing parameter "{k}" with best value {b} '
+                                         f'from trial {t.number}')
+                            ps[k] = b
+                        else:
+                            logging.info(f'No trials in study {s.study_name} '
+                                         f'- removing parameter {k} from fixed_params')
+                            ps.pop(k)
+                    else:
+                        logging.info(f'Fixing parameter "{k}" with value {v}')
             sampler = getattr(optuna.samplers, self.sampler)(**self.sampler_kwargs)
-        else:
-            sampler = None
+            if self.sampler == 'PartialFixedSampler':
+                self.sampler_kwargs['base_sampler'] = bs_str
+                self.sampler_kwargs['base_sampler_kwargs'] = ks
+        return sampler
+
+    def initialize_pruner(self):
+        pruner = None
         if self.pruner is not None:
+            if self.pruner == 'PatientPruner':  # Initialize wrapped sampler
+                if 'wrapped_pruner' in self.pruner_kwargs:
+                    wp_str = self.pruner_kwargs['wrapped_pruner']
+                else:
+                    logging.warning('wrapped_pruner is not in pruner_kwargs '
+                                    '- using None pruner')
+                    wp_str = None
+                if 'wrapped_pruner_kwargs' in self.pruner_kwargs:
+                    ks = self.pruner_kwargs.pop('wrapped_pruner_kwargs')
+                else:
+                    logging.warning(f'wrapped_pruner_kwargs is not in pruner_kwargs '
+                                    f'- using default parameters of {wp_str}')
+                    ks = {}
+                if wp_str is not None:
+                    wp = getattr(optuna.pruners, wp_str)(**ks)
+                else:
+                    wp = None
+                self.pruner_kwargs['wrapped_pruner'] = wp
             pruner = getattr(optuna.pruners, self.pruner)(**self.pruner_kwargs)
-        else:
-            pruner = None
+            if self.pruner == 'PatientPruner':
+                self.pruner_kwargs['wrapped_pruner'] = wp_str
+                self.pruner_kwargs['wrapped_pruner_kwargs'] = ks
+        return pruner
+
+    def create_study(self):
+        sampler = self.initialize_sampler()
+        pruner = self.initialize_pruner()
         if self.do_create_study:
             if self.do_read_study:
                 load_if_exists = True  # Raise exception if exists
@@ -454,7 +550,8 @@ class Optuna(Optimize):
                             labels=labels)
                         if color is not None:
                             fig.layout.coloraxis.colorbar.title = color
-                        fig.write_html(p / f"pareto_front-{'-'.join(f'{n}-{d}' for n, d in zip(c_ns, c_ds))}.html")
+                        fig.write_html(
+                            p / f"pareto_front-{'-'.join(f'{n}-{d}' for n, d in zip(c_ns, c_ds))}.html")
                     for c in combinations(range(n), 3):
                         c_vs = [old2new[f'values_{x}'] for x in c]
                         c_ns = [ns[x] for x in c]
@@ -469,29 +566,35 @@ class Optuna(Optimize):
                             labels=labels)
                         if color is not None:
                             fig.layout.coloraxis.colorbar.title = color
-                        fig.write_html(p / f"pareto_front-{'-'.join(f'{n}-{d}' for n, d in zip(c_ns, c_ds))}.html")
+                        fig.write_html(
+                            p / f"pareto_front-{'-'.join(f'{n}-{d}' for n, d in zip(c_ns, c_ds))}.html")
                 except Exception as e:
                     print(e)
-        if self.do_plot_other:
-            for i, (n, d) in enumerate(self.objectives.items()):
-                try:  # required plotly
+        for i, (n, d) in enumerate(self.objectives.items()):
+            try:  # required plotly
+                if self.do_plot_history:
                     optuna.visualization.plot_optimization_history(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
                         p / f"optimization_history-{d}-{n}.html")
+                if self.do_plot_slice:
                     optuna.visualization.plot_slice(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
                         p / f"slice-{d}-{n}.html")
+                if self.do_plot_contour:
                     optuna.visualization.plot_contour(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
                         p / f"contour-{d}-{n}.html")
+                if self.do_plot_parallel:
                     optuna.visualization.plot_parallel_coordinate(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
                         p / f"parallel_coordinate-{d}-{n}.html")
+                if self.do_plot_edf:
                     optuna.visualization.plot_edf(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
                         p / f"edf-{d}-{n}.html")
-                except Exception as e:
-                    print(e)
+            except Exception as e:
+                print(e)
+            if self.do_plot_importance:
                 try:  # required sklearn
                     optuna.visualization.plot_param_importances(
                         study, target_name=n, target=lambda x: x.values[i]).write_html(
